Intestinal surgery data involving 844 cancer patients. The data consists of pairs (\(n_i\), \(s_i\)) where \(n_i\) is the number of satellites removed and \(s_i\) is the number of satellites found to be malignant.
Source:R/deconvolveR-package.R
surg.RdIntestinal surgery data involving 844 cancer patients. The data consists of pairs (\(n_i\), \(s_i\)) where \(n_i\) is the number of satellites removed and \(s_i\) is the number of satellites found to be malignant.
Usage
data(surg)